@@ -48,8 +48,22 @@ class CAISOClient(BaseClient):
         self.handle_options(data='gen', latest=latest, yesterday=yesterday,
                             start_at=start_at, end_at=end_at, **kwargs)
 
+        # ensure market and freq are set
+        if 'market' not in self.options:
+            if self.options['forecast']:
+                self.options['market'] = self.MARKET_CHOICES.dam
+            else:
+                self.options['market'] = self.MARKET_CHOICES.fivemin
+        if 'freq' not in self.options:
+            if self.options['forecast']:
+                self.options['freq'] = self.FREQUENCY_CHOICES.hourly
+            else:
+                self.options['freq'] = self.FREQUENCY_CHOICES.fivemin
+
         if latest:
             return self._generation_latest()
+        elif self.options['forecast']:
+            return self._generation_forecast()
         else:
             return self._generation_historical()
 
@@ -97,6 +111,50 @@ class CAISOClient(BaseClient):
             # return all data
             return parsed_data
 
+    def get_trade(self, latest=False,
+                       start_at=False, end_at=False, **kwargs):
+        # set args
+        self.handle_options(data='trade', latest=latest,
+                            start_at=start_at, end_at=end_at, **kwargs)
+
+        # ensure market and freq are set
+        if 'market' not in self.options:
+            if self.options['forecast']:
+                self.options['market'] = self.MARKET_CHOICES.dam
+            else:
+                self.options['market'] = self.MARKET_CHOICES.fivemin
+        if 'freq' not in self.options:
+            if self.options['forecast']:
+                self.options['freq'] = self.FREQUENCY_CHOICES.hourly
+            else:
+                self.options['freq'] = self.FREQUENCY_CHOICES.fivemin
+
+        # construct and execute OASIS request
+        payload = self.construct_oasis_payload('ENE_SLRS')
+        oasis_data = self.fetch_oasis(payload=payload)
+
+        # parse data
+        parsed_data = self.parse_oasis_slrs(oasis_data)
+
+        if self.options['latest']:
+            # select latest
+            latest_dp = None
+            latest_ts = self.utcify('1900-01-01 12:00')
+            now = self.utcify(datetime.utcnow(), tz_name='utc')
+            for dp in parsed_data:
+                if dp['timestamp'] < now and dp['timestamp'] > latest_ts:
+                    latest_dp = dp
+                    latest_ts = dp['timestamp']
+
+            # return latest
+            if latest_dp:
+                return [latest_dp]
+            else:
+                return []
+        else:
+            # return all data
+            return parsed_data
+
     def construct_oasis_payload(self, queryname, **kwargs):
         # get start and end times
         if self.options['latest']:
@@ -244,11 +302,14 @@ class CAISOClient(BaseClient):
                 continue
             
         # collect values into dps
+        freq = self.options.get('freq', self.FREQUENCY_CHOICES.hourly)
+        market = self.options.get('market', self.MARKET_CHOICES.hourly)
+
         for ts, preparsed_dp in preparsed_data.iteritems():
             # set up base
             base_parsed_dp = {'timestamp': ts,
-                              'freq': self.FREQUENCY_CHOICES.hourly,
-                              'market': self.MARKET_CHOICES.hourly,
+                              'freq': freq,
+                              'market': market,
                               'gen_MW': 0, 'ba_name': self.NAME}
                           
             # collect data
@@ -265,22 +326,34 @@ class CAISOClient(BaseClient):
         """Parse raw data output of fetch_oasis for System Load and Resource Schedules."""
         # set up storage
         parsed_data = []
+
+        # set strings to search on
+        if self.options['data'] == 'gen':
+            data_item_map = {'ISO_TOT_GEN_MW': 'gen_MW'}
+        elif self.options['data'] == 'trade':
+            data_item_map = {'ISO_TOT_EXP_MW': 'exp_MW',
+                            'ISO_TOT_IMP_MW': 'imp_MW'}
+        else:
+            data_item_map = {}
+
+        freq = self.options.get('freq', self.FREQUENCY_CHOICES.fivemin)
+        market = self.options.get('market', self.MARKET_CHOICES.fivemin)
         
         # extract values from xml
         for raw_soup_dp in raw_data:
-            if raw_soup_dp.find('data_item').string == 'ISO_TOT_GEN_MW':
+            data_item = raw_soup_dp.find('data_item').string
+            if data_item in data_item_map.keys():
                 
                 # parse timestamp
                 ts = self.utcify(raw_soup_dp.find('interval_start_gmt').string)
 
-                # set up base
-                parsed_dp = {'timestamp': ts, 'fuel_name': 'other',
-                              'freq': self.FREQUENCY_CHOICES.fivemin,
-                              'market': self.MARKET_CHOICES.fivemin,
-                              'ba_name': self.NAME}
-                    
-                # store generation value
-                parsed_dp['gen_MW'] = float(raw_soup_dp.find('value').string)
+                # assemble data
+                parsed_dp = {data_item_map[data_item]: float(raw_soup_dp.find('value').string)}
+                parsed_dp.update({'timestamp': ts, 'freq': freq, 'market': market, 'ba_name': self.NAME})
+                if self.options['data'] == 'gen':
+                    parsed_dp.update({'fuel_name': 'other'})
+
+                # add to storage
                 parsed_data.append(parsed_dp)
                 
         # return
@@ -403,3 +476,44 @@ class CAISOClient(BaseClient):
 
         # no matching OASIS data found, so return null
         return []
+
+    def _generation_forecast(self, **kwargs):
+        # set up
+        parsed_data = []
+
+        # get OASIS total gen data
+        gen_payload = self.construct_oasis_payload(queryname='ENE_SLRS', schedule='ALL')
+        gen_oasis_data = self.fetch_oasis(payload=gen_payload)
+        gen_dps = self.parse_oasis_slrs(gen_oasis_data)
+
+        # get OASIS renewable gen data
+        ren_payload = self.construct_oasis_payload(queryname='SLD_REN_FCST')
+        ren_oasis_data = self.fetch_oasis(payload=ren_payload)
+        ren_dps = self.parse_oasis_renewable(ren_oasis_data)
+
+        # set of times with both gen and renewable data
+        times = set([dp['timestamp'] for dp in ren_dps]) & set([dp['timestamp'] for dp in gen_dps])
+
+        # handle renewables
+        total_ren_MW = {}
+        for dp in ren_dps:
+            if dp['timestamp'] in times:
+                # assemble renewable totals for each time
+                try:
+                    total_ren_MW[dp['timestamp']] += dp['gen_MW']
+                except KeyError:
+                    total_ren_MW[dp['timestamp']] = dp['gen_MW']
+
+                # add to storage
+                parsed_data.append(dp)
+
+        # handle generation
+        for dp in gen_dps:
+            if dp['timestamp'] in times:
+                # subtract off renewable totals
+                dp['gen_MW'] -= total_ren_MW[dp['timestamp']]
+                # add to storage
+                parsed_data.append(dp)
+        
+        # return
+        return parsed_data
